$(function () {
  'use strict';

  var console = window.console || { log: function () {} };
  var URL = window.URL || window.webkitURL;
  var $image = $('#image');
  var $download = $('#download');
  var $dataX = $('#dataX');
  var $dataY = $('#dataY');
  var $dataHeight = $('#dataHeight');
  var $dataWidth = $('#dataWidth');
  var $dataRotate = $('#dataRotate');
  var $dataScaleX = $('#dataScaleX');
  var $dataScaleY = $('#dataScaleY');

  //0、初始裁剪，这里图片裁剪的高度宽度请先设置好
  var cropperW = 200;
  var cropperH = 150;
  var aspectRatio;
  if(cropperW <= 0 || typeof(cropperW) == 'undefined' || cropperW == null || cropperH <= 0 || typeof(cropperH) == 'undefined' || cropperH == null){
    aspectRatio = NaN;
  } else {
    aspectRatio = cropperW / cropperH;
  }

  //配置参数
  var options = {
    viewMode: 1,
    autoCropArea: 0.7, //初始裁剪区域占图片比例
    aspectRatio: aspectRatio, //裁剪比例，NaN-自由选择区域
    preview: '.img-preview', //不同尺寸预览区
    crop: function (e) {
      $dataX.val(Math.round(e.detail.x));
      $dataY.val(Math.round(e.detail.y));
      $dataHeight.val(Math.round(e.detail.height));
      $dataWidth.val(Math.round(e.detail.width));
      $dataRotate.val(e.detail.rotate);
      $dataScaleX.val(e.detail.scaleX);
      $dataScaleY.val(e.detail.scaleY);
    }
  };
  var originalImageURL = $image.attr('src');
  var uploadedImageName = 'cropped.jpg';
  var uploadedImageType = 'image/jpeg';
  var uploadedImageURL;

  // Tooltip
  $('[data-toggle="tooltip"]').tooltip();

  //1、弹窗
  var cropper_index = layer.open({
        title:'',
        closeBtn:false,
        type:1,
        content:$('#cropperimg'),
        area:['100%','100%'],
        maxmin:false
    });

  //2、裁剪初始化
  $image.on({
    ready: function (e) {
      console.log(e.type);
    },
    cropstart: function (e) {
      console.log(e.type, e.detail.action);
    },
    cropmove: function (e) {
      console.log(e.type, e.detail.action);
    },
    cropend: function (e) {
      console.log(e.type, e.detail.action);
    },
    crop: function (e) {
      console.log(e.type);
    },
    zoom: function (e) {
      console.log(e.type, e.detail.ratio);
    }
  }).cropper(options);

  //3、渲染初始化
  var reader = new FileReader();
  reader.readAsDataURL(file);//发起异步请求
  reader.onload = function(){
      // 通过 reader.result 来访问生成的 DataURL
      var url = reader.result;
      //选择图片后重新初始裁剪区
      $image.cropper('reset', true).cropper('replace', url);
  }

  //4、取消裁剪
  $('#coloseCropper').click(function () {
      layer.close(cropper_index);
      $image.cropper('reset', true);
      return false;
  });

  // 按钮
  if (!$.isFunction(document.createElement('canvas').getContext)) {
    $('button[data-method="getCroppedCanvas"]').prop('disabled', true);
  }

  if (typeof document.createElement('cropper').style.transition === 'undefined') {
    $('button[data-method="rotate"]').prop('disabled', true);
    $('button[data-method="scale"]').prop('disabled', true);
  }

  // 下载
  /*if (typeof $download[0].download === 'undefined') {
    $download.addClass('disabled');
  }*/

  // 选项
  $('.docs-toggles').on('change', 'input', function () {
    var $this = $(this);
    var name = $this.attr('name');
    var type = $this.prop('type');
    var cropBoxData;
    var canvasData;

    if (!$image.data('cropper')) {
      return;
    }

    if (type === 'checkbox') {
      options[name] = $this.prop('checked');
      cropBoxData = $image.cropper('getCropBoxData');
      canvasData = $image.cropper('getCanvasData');

      options.ready = function () {
        $image.cropper('setCropBoxData', cropBoxData);
        $image.cropper('setCanvasData', canvasData);
      };
    } else if (type === 'radio') {
      options[name] = $this.val();
    }

    //销毁
    $image.cropper('destroy');

    //3、渲染初始化
    var reader = new FileReader();
    reader.readAsDataURL(file);//发起异步请求
    reader.onload = function(){
        // 通过 reader.result 来访问生成的 DataURL
        var url = reader.result;
        //选择图片后重新初始裁剪区
        $image.cropper('reset', true).cropper('replace', url);
    }
  });

  // 方法
  $('.docs-buttons').on('click', '[data-method]', function () {
    var $this = $(this);
    var data = $this.data();
    var cropper = $image.data('cropper');
    var cropped;
    var $target;
    var result;

    if ($this.prop('disabled') || $this.hasClass('disabled')) {
      return;
    }

    if (cropper && data.method) {
      data = $.extend({}, data); // 复制

      if (typeof data.target !== 'undefined') {
        $target = $(data.target);

        if (typeof data.option === 'undefined') {
          try {
            data.option = JSON.parse($target.val());
          } catch (e) {
            console.log(e.message);
          }
        }
      }

      cropped = cropper.cropped;

      switch (data.method) {
        case 'rotate':
          if (cropped && options.viewMode > 0) {
            $image.cropper('clear');
          }

          break;

        case 'getCroppedCanvas':
          if (uploadedImageType === 'image/jpeg') {
            if (!data.option) {
              data.option = {};
            }

            data.option.fillColor = '#fff';
          }

          break;
      }

      result = $image.cropper(data.method, data.option, data.secondOption);

      switch (data.method) {
        case 'rotate':
          if (cropped && options.viewMode > 0) {
            $image.cropper('crop');
          }

          break;

        case 'scaleX':
        case 'scaleY':
          $(this).data('option', -data.option);
          break;

        case 'getCroppedCanvas':
          //5、监听裁剪成功
          if (result) {
            var src = result.toDataURL(uploadedImageType);
            layer.close(cropper_index);
            
            //转成文件类型
            var newFile = dataURLtoFile(src, file.name);

            //修改原队列的File上传源
            newFile.queueItem = file.queueItem;
            file.queueItem.data('file', newFile);

            //转成blob
            //var type = $image.attr('src').split(';')[0].split(':')[1];
            /*result.toBlob(function(blob) {

              console.log(blob);
              blob.name = file.name;
              blob.queueItem = file.queueItem;
              file.queueItem.data('file', blob);
              console.log(file);

              //to do ....... 这垃圾玩意
            }, type);     */       
          }

          break;

        case 'destroy':
          if (uploadedImageURL) {
            URL.revokeObjectURL(uploadedImageURL);
            uploadedImageURL = '';
            $image.attr('src', originalImageURL);
          }

          break;
      }

      if ($.isPlainObject(result) && $target) {
        try {
          $target.val(JSON.stringify(result));
        } catch (e) {
          console.log(e.message);
        }
      }
    }
  });

  // 按键
  $(document.body).on('keydown', function (e) {
    if (e.target !== this || !$image.data('cropper') || this.scrollTop > 300) {
      return;
    }

    switch (e.which) {
      case 37:
        e.preventDefault();
        $image.cropper('move', -1, 0);
        break;

      case 38:
        e.preventDefault();
        $image.cropper('move', 0, -1);
        break;

      case 39:
        e.preventDefault();
        $image.cropper('move', 1, 0);
        break;

      case 40:
        e.preventDefault();
        $image.cropper('move', 0, 1);
        break;
    }
  });

  // 上传图片
  var $inputImage = $('#inputImage');

  if (URL) {
    $inputImage.change(function () {
      var files = this.files;
      var file;

      if (!$image.data('cropper')) {
        return;
      }

      if (files && files.length) {
        file = files[0];

        if (/^image\/\w+$/.test(file.type)) {
          uploadedImageName = file.name;
          uploadedImageType = file.type;

          if (uploadedImageURL) {
            URL.revokeObjectURL(uploadedImageURL);
          }

          uploadedImageURL = URL.createObjectURL(file);
          $image.cropper('destroy').attr('src', uploadedImageURL).cropper(options);
          $inputImage.val('');
        } else {
          window.alert('请选择图片文件');
        }
      }
    });
  } else {
    $inputImage.prop('disabled', true).parent().addClass('disabled');
  }
});

/**
 * [dataURLtoBlob base64转为Blob]
 * @param  {[type]} dataurl [image地址]
 * @return {[type]}         [description]
 */
function dataURLtoBlob(dataurl) {
    var arr = dataurl.split(','), mime = arr[0].match(/:(.*?);/)[1],
        bstr = atob(arr[1]), n = bstr.length, u8arr = new Uint8Array(n);
    while (n--) {
        u8arr[n] = bstr.charCodeAt(n);
    }
    return new Blob([u8arr], { type: mime });
}


/**
 * [blobToDataURL Blob转为base64]
 * @param  {[type]}   blob     [blob对象]
 * @param  {Function} callback [回调]
 * @return {[type]}            [description]
 */
function blobToDataURL(blob, callback) {
    let a = new FileReader();
    a.onload = function (e) { callback(e.target.result); }
    a.readAsDataURL(blob);
}

/**
 * [dataURLtoFile base64转换为file]
 * @param  {[type]} dataurl  [description]
 * @param  {[type]} filename [description]
 * @return {[type]}          [description]
 */
function dataURLtoFile(dataurl, filename) {
    var arr = dataurl.split(','), mime = arr[0].match(/:(.*?);/)[1],
        bstr = atob(arr[1]), n = bstr.length, u8arr = new Uint8Array(n);
    while(n--){
        u8arr[n] = bstr.charCodeAt(n);
    }
    return new File([u8arr], filename, {type:mime});
}

/**
 * [getImgToBase64 js图片转换为base64]
 * @param  {[type]}   url      [description]
 * @param  {Function} callback [description]
 * @return {[type]}            [description]
 */
function getImgToBase64(url, callback){
  var canvas = document.createElement('canvas'),
    ctx = canvas.getContext('2d'),
    img = new Image;
  img.crossOrigin = 'Anonymous';
  img.onload = function(){
    canvas.height = img.height;
    canvas.width = img.width;
    ctx.drawImage(img,0,0);
    var dataURL = canvas.toDataURL('image/png');
    callback(dataURL);
    canvas = null;
  };
  img.src = url;
}